@ECHO OFF
SETLOCAL ENABLEDELAYEDEXPANSION

ECHO Simple Mod Manager, Version 1.3.1, by Quantum at saintsrowmods.com

:again
CALL :SetFullPath modsDir .
CALL :SetFullPath srDir ..
CALL :SetFullPath logFile smm.log
SET logFile="%logFile%"
IF EXIST smmcfg.bat CALL smmcfg.bat

ECHO. 
ECHO MOD LIST:
CALL :ListMods
SET ch=0
SET /P ch=Type a mod number to install/remove (or Q to quit): 
IF /I %ch%==q GOTO :EOF
CALL :GetDir %ch%
IF "%dir%"=="" (
  ECHO Invalid selection.  Please enter one of the numbers in the mod list.
  GOTO again
)
CALL :CheckInstalled "%dir%"
IF %installed%==1 (
  CALL :RemoveMod "%dir%"
  GOTO again
)
CALL :CanInstall "%dir%"
IF %allowed%==1 (
  CALL :InstallMod "%dir%"
) ELSE (
  ECHO ERROR: Can't Install -- Mod conflicts with existing files.
)
GOTO again

REM Sets an environment variable to a fully qualified path name.
REM Usage: CALL :SetFullPath <var name> "<path>"
:SetFullPath
SET %1=%~f2
GOTO :EOF

REM Lists the mods and their current installed status.
REM Usage: CALL :ListMods
:ListMods
SET n=1
FOR /D %%I IN ("%modsDir%\*") DO (
  REM ECHO %%~nI
  CALL :CheckInstalled "%%~nxI"
  IF !installed!==1 (
    ECHO !n! [INSTALLED] %%~nxI
  ) ELSE (
    ECHO !n! [---------] %%~nxI
  )
  SET /A n=!n!+1
)
GOTO :EOF

REM Gets the mod subdirectory name given the menu number.
REM Usage:  CALL :GetDir <menu number>
REM Result: %dir% = mod subdirectory name.
:GetDir
SET n=1
SET dir=
FOR /D %%I IN ("%modsDir%\*") DO (
  IF !n!==%1 (
    SET dir=%%~nxI
    GOTO :EOF
  )
  SET /A n=!n!+1
)
GOTO :EOF

REM Tests if the mod subdirectory is installed.
REM Usage:  CALL :CheckInstalled "<mod subdirectory name>"
REM Result: %installed% = 1 if installed, 0 if not installed.
:CheckInstalled
SET installed=0
FOR %%D IN ("" "tables\") DO FOR %%F IN ("%modsDir%\%~1\%%~D*") DO (
  IF NOT EXIST "%srDir%\%%~D%%~nxF" GOTO :EOF
  SET file="%srDir%\%%~D%%~nxF"
  CALL :GetAttr %%file%%
  IF NOT "%%~tF"=="!time!" GOTO :EOF
  IF NOT "%%~zF"=="!size!" GOTO :EOF
)
SET installed=1
GOTO :EOF

REM Tests if the mod subdirectory can be installed without overwriting anything.
REM Usage:  CALL :CanInstall "<mod subdirectory name>"
REM Result: %allowed% = 1 if can be installed, 0 if cannot be installed.
:CanInstall
SET allowed=0
FOR %%D IN ("" "tables\") DO FOR %%F IN ("%modsDir%\%~1\%%~D*") DO (
  IF EXIST "%srDir%\%%~D%%~nxF" GOTO :EOF
)
SET allowed=1
GOTO :EOF

REM Gets the date/time and size of a file.
REM Usage:  CALL :GetAttr "<file path>"
REM Result: %time% = file date/time string
REM         %size% = file size in bytes
:GetAttr
SET time=%~t1
SET size=%~z1
GOTO :EOF

REM Installs a mod.
REM Usage: CALL :InstallMod "<mod subdirectory name>"
:InstallMod
ECHO. 
ECHO Installing mod "%~1"...
ECHO. >>%logFile%
ECHO -------------------- >>%logFile%
ECHO %date% %time% >>%logFile%
ECHO Installing mod "%~1" >>%logFile%
CALL :CallScript "%~1" install1
IF EXIST "%modsDir%\%~1\tables" IF NOT EXIST "%srDir%\tables" (
  ECHO MKDIR "%srDir%\tables" >>%logFile%
  MKDIR "%srDir%\tables" 1>>%logFile% 2>&1
)
FOR %%D IN ("" "tables\") DO (
  DIR "%modsDir%\%~1\%%~D*" /A-D /B >NUL 2>NUL
  IF NOT ERRORLEVEL 1 ( 
    ECHO COPY /Y "%modsDir%\%~1\%%~D*" "%srDir%\%%~D" >>%logFile%
    COPY /Y "%modsDir%\%~1\%%~D*" "%srDir%\%%~D" 1>>%logFile% 2>&1
    IF ERRORLEVEL 1 GOTO Failed
  )
)
CALL :CallScript "%~1" install2
ECHO Done.
GOTO :EOF

REM Removes a mod.
REM Usage: CALL :RemoveMod "<mod subdirectory name>"
:RemoveMod
ECHO. 
ECHO Removing mod "%~1"...
SET sure=n
SET /P sure=Are you sure? (y/n) 
IF /I NOT "%sure:~0,1%"=="y" GOTO :EOF
ECHO. >>%logFile%
ECHO -------------------- >>%logFile%
ECHO %date% %time% >>%logFile%
ECHO Removing mod "%~1" >>%logFile%
CALL :CallScript "%~1" remove1
FOR %%D IN ("" "tables\") DO FOR %%F IN ("%modsDir%\%~1\%%~D*") DO (
  ECHO DEL "%srDir%\%%~D%%~nxF" >>%logFile%
  DEL "%srDir%\%%~D%%~nxF" 1>>%logFile% 2>&1
  IF ERRORLEVEL 1 GOTO Failed
)
CALL :CallScript "%~1" remove2
ECHO Done.
GOTO :EOF

REM Displays a "Failed" message.
:Failed
ECHO Failed.
GOTO :EOF

REM Calls an external script if it exists.
REM Usage: CALL :CallScript "<mod subdirectory name>" <script name>
:CallScript
IF NOT EXIST "%~1\scripts\%~2.bat" GOTO :EOF
PUSHD "%~1\scripts"
CALL "%~2.bat"
POPD
GOTO :EOF
